% WHG
%
%
% Aim: To reconstruct EEG file after removing artifactual sources.
% Written by: Manish Saggar (manish.saggar@gmail.com).
%
% Copyright (C) 2011 Manish Saggar
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% 
% This tool is written as an attachment to SMART tool for EEG
% pre-processing. For more information on the SMART tool. Please see the following
% references. Also, if you use this tool in your research, please cite
% these as well.
% (1) Saggar M, et al. (2012) Intensive training induces longitudinal changes 
%      in meditation state-related EEG oscillatory activity. 
%      Front. Hum. Neurosci. 6:256. doi: 10.3389/fnhum.2012.00256 
% (2) Saggar, M. (2011). Computational analysis of meditation. 
%      Retrieved from University of Texas Digital Repository: 
%      Electronic Theses and Dissertations, (Austin, TX), Pages: 24-44.
%      Link: http://www.stanford.edu/~saggar/pubs/Dissertation_Saggar2011_Meditation.pdf
%
% Input: (a) Original EEG data from BESA (in dat file)
%        (b) Mixing matrix saved in .mat file format
%        (c) Artifacts selected using SMART
%
% Output: Generates the ReconEEG_<originalEEGfile>.dat to be opened in BESA
% or EEGLAB or Fieldtrip for futher analysis.
%
% Usage: reconstructEEGafterSMART(originalEEGfile, savedMixingMatrix, artifact_sources)
% 
function reconstructEEGafterSMART(originalEEGfile, savedMixingMatrix, artifact_sources)
    [path, name, ext, ver] = fileparts(originalEEGfile);
    new_full_path_eeg_file = strcat(path, '/','ReconEEG','_', name, ext);
    W = load(savedMixingMatrix);
    A = inv(W);
    hdr = readBESAsb_header(originalEEGfile);
    t_p = 60;  % in seconds; reading 1 min of data every time. Change this depending upon how much you need to read.
    nsamples_to_read = hdr.Fs * t_p;
    end_min = floor(hdr.nSamples/nsamples_to_read);
    for i = 1:nsamples_to_read:end_min*nsamples_to_read
        X = readBESAsb_data(originalEEGfile, hdr, i, i+nsamples_to_read-1);
        S = W*X;
        A_new = [];
        A_new = [A_new; A(:,(setdiff([1:hdr.nChans],artifact_sources)))]; %#ok<AGROW>
        S_new = [];
        S_new = [S_new; S(setdiff([1:hdr.nChans],artifact_sources),:)]; %#ok<AGROW>
        X_cap = A_new*S_new;
        writeBESAsb_data(X_cap, new_full_path_eeg_file);
    end
    % additionally read the extra partial minute, that is not read above.
    extra_samples = nsamples_to_read*(hdr.nSamples/nsamples_to_read - floor(hdr.nSamples/nsamples_to_read));
        X = readBESAsb_data(originalEEGfile, hdr, i, i+extra_samples-1);
        S = W*X;
        A_new = [];
        A_new = [A_new; A(:,(setdiff([1:hdr.nChans],artifact_sources)))];
        S_new = [];
        S_new = [S_new; S(setdiff([1:hdr.nChans],artifact_sources),:)];
        X_cap = A_new*S_new;
        writeBESAsb_data(X_cap, new_full_path_eeg_file);
    
    % writting the new generic file, and copying the evt file as it is, so
    % that BESA can use the new data appropriately.
    writeBESAgeneric(new_full_path_eeg_file, hdr);
end
% Following four functions were adapted from Matlab Scripts provided by
% BESA software toolkit. See here more details - http://www.besa.de/updates/matlab/
function writeBESAsb_data(X_cap, new_file)
    fid = fopen(new_file, 'a','ieee-le');
    if fid == -1
       fprintf(1,'In writeBESAsb_data:: Output file not found\n');
       exit;
    end
    fwrite(fid, X_cap, 'float32');
    fclose(fid);
end

function writeBESAgeneric(new_file, hdr)
    [path, name, ext, ver] = fileparts(new_file);
    fid = fopen(strcat(path, '/', name, '.generic'), 'w');
    fprintf(fid,'BESA Generic Data\n\n');
    fprintf(fid,'nChannels=%i\n\n', hdr.nChans);
    fprintf(fid,'sRate=%f\n\n', hdr.Fs);
    fprintf(fid,'nSamples=%i\n\n', hdr.nSamples);
    fprintf(fid,'format=float\n\n');
    fprintf(fid,'file=%s\n\n', strcat(name,ext));
    fclose(fid);
    
    % event file copied as it is.
    cmd = strcat({'!cp '}, {strcat(path, '/', name(7:end), '.evt')}, {' '} , {strcat(' ', path, '/', name, '.evt')});
    eval(cmd{1});
end
function hdr = readBESAsb_header(filename)
    fid=fopen([filename(1:end-4),'.generic'],'r');
    if fid==-1
      fid=fopen([filename(1:end-4),'.gen'],'r');
    end

    fscanf(fid,'BESA Generic Data\n');
    nChannels = fscanf(fid,'nChannels=%i\n');
    sRate = fscanf(fid,'sRate=%f\n');
    nSamples = fscanf(fid,'nSamples=%i\n');
    format = fscanf(fid,'format=%s');
    file = fscanf(fid,'\nfile=%s');
    prestimulus = fscanf(fid,'prestimulus=%f\n');
    epochs = fscanf(fid,'epochs=%i\n');
    fclose(fid);

    hdr.Fs = sRate;
    hdr.nChans = nChannels ;
    hdr.nSamples = nSamples;
    hdr.nSamplesPre = prestimulus;
    hdr.nTrials = epochs;
end
function data = readBESAsb_data(filename, hdr, beg_sample, end_sample)
    if isempty(findstr(filename,'.dat'))
      filename = [filename,'.dat'];
    end
    % Bytes to skip per sample, is simply equal to
    % 4xnChannels
    bytes_per_sample = 4*hdr.nChans;
    fid=fopen(filename, 'r', 'ieee-le');
    % Skippping
    fseek(fid, bytes_per_sample*(beg_sample-1),-1);
    data=fread(fid,[hdr.nChans,(end_sample-beg_sample+1)],'float32');
    fclose(fid);
end


