% WHG
%
% Aim: To create web-based artifact review tool (SMART) for CONTINUOUS EEG data.
% Written by: Manish Saggar (manish.saggar@gmail.com).
%
% Copyright (C) 2011  Manish Saggar
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% 
% For more information on the SMART tool. Please see the following
% references. Also, if you use this tool in your research, please cite
% these as well.
% (1) Saggar M, et al. (2012) Intensive training induces longitudinal changes 
%      in meditation state-related EEG oscillatory activity. 
%      Front. Hum. Neurosci. 6:256. doi: 10.3389/fnhum.2012.00256 
% (2) Saggar, M. (2011). Computational analysis of meditation. 
%      Retrieved from University of Texas Digital Repository: 
%      Electronic Theses and Dissertations, (Austin, TX), Pages: 24-44.
%      Link: http://www.stanford.edu/~saggar/pubs/Dissertation_Saggar2011_Meditation.pdf
%
% Input: (a) Source Time series (in component space; NumSources x length of experiment)
%        (b) Power spectra estimation of source trials (of each trial; NumSources x length(frequencies))
%        (c) Frequencies on which power spectr was estimated 
%        (d) Unmixing matrix (W; numChannels x numChannels)
%        (e) Topographic locations of channels (sfpfile)
%        (f) Output file name (ofilename)
%        (g) Prefix for any conditions (cond_prefix)
%
% Output: Generates a folder with a HTML file and the supporting files in
% it.
%
% Usage: smartEEGcontinuous(aST, pST, freq, W, sfpfile, ofilename, cond_prefix)
%
%
% NOTE 1: The generated ArtifactsHTML.html file expects that the poll.cgi
% file exists on the localhost webserver directory. I have included one
% such perl script (poll.cgi) on the webpage. poll.cgi needs to go under
% /Library/WebServer/CGI-Executables/ (on a Mac, not sure about the PC) and the webserver
% (localhost) should be switched "on" for this to work. Also inside
% poll.cgi file, one needs to specigy numEEGCh parameter. 
%
% NOTE 2: After you recognize the sources of artifacts (EMG, EOG etc.) and
% clicked on Vote button in the ArtifactsHTML.html file. The perl script
% poll.cgi will create a file with source classification in it. Usually the
% location of this file will be in /Library/WebServer/Documents (on a
% Mac). One can then select the artifactual sources and reconstruct
% original data using the the inverse of mixing matrix W. For example, if
% a set named artifacts contains a list of artifactual sources (say
% [1,2,3]), then we can reconstruct data in sensor space without these
% artifacts as follows, X_recon = A(:,setdiff(allsources, artifacts)) *
% St(setdiff(allsources, artifacts),:). Where A = inv (W) and St = source
% time series. See attached reconstructEEGafterSMART.m file that does this
% step for you.
%  
function smartEEGcontinuous(aST, pST, freq, W, sfpfile, ofilename, cond_prefix)
    % Global thresholds for classifying a source as neural or non-neural.
    % Please play with these parameters and they could vary based on the
    % dataset at hand.
    emgThreshold = 0.6; % lower cut-off of Autocorrelation function to denote a source as EMG
    eogThreshold = 0.7; % lower cut-off of Autocorrelation function to denote a source as EOG
    el = readlocs(sfpfile); % this function is from EEGLAB. Make sure you have this installed.
    blockSize = 2048; % sampling rate or size of fake-epoch 
    
    % Calculate EMG and EOG components based on Auto-correlation function on the
    % evoked (or average) source trial.
    [emgnew, ac] = findACFbasedEMG(aST, W, emgThreshold, blockSize);     
    eognew = findACFbasedEOG(ac, W, eogThreshold);
   
    % Calculate Peak components based on PowerSpectrum analysis on each
    % Source trial. 
    [peaksnew peaks]= findPeaks(pST, W, freq);  
    
    % Creating HTML based SMART tool for further processing. 
    printHTML(emgnew, eognew, peaksnew, peaks, ac, el, W, pST, aST, ofilename, cond_prefix, freq, blockSize);    

end
function [y acSt] = findACFbasedEMG(St, W, thresh, blockSize)
    y = [];
    acSt = [];
    % only 64 delays. Change this if you want less or more delays to calulate ACF
    acf_delay = 64;
    
    for s = 1:1:size(W,1)
        acst = zeros(acf_delay,1); 
        counter = 0;
        for t = 1:blockSize:size(St,2)
             acst = acst + autocorr(St(s, round(t):round(t)+blockSize-1)',acf_delay); 
             counter = counter + 1;
        end
        % average the acf for all trials for each source.
        acSt(s,:) = (acst./counter)';
        if min(acSt(s,1:20)) < thresh  % trying min instead of mean
            y = [y s];
        end
    end
end
function [y] = findACFbasedEOG(ac, W, thresh)
    y = [];
    for i = 1:1:size(W,1)
       if mean(ac(i,:)) > thresh
           y = [y i];
       end
    end
end
function [y pp] = findPeaks(pST, W, freq)
    y =[];
    pp = [];
    psAmpthresh = 1e-9;   % parameter has to be changed based on the baseline of data.  
    foi = 60;  % Only looking for 60Hz power line source.
    for s = 1:1:size(W,1)
        if max(pST(s,:)) < psAmpthresh 
            continue;
        end
        [p,l] = findpeaks(pST(s,:),'SORTSTR','descend','MINPEAKHEIGHT',psAmpthresh);
        npeaks = 5;
        if length(l) < 1
            continue;
        elseif length(l) < 5
            npeaks = length(l);
        end
        freqPeaks = round(freq(l));
        for i = 1:1:npeaks   % just look at first three peaks
           if freqPeaks(i) == foi-1 || freqPeaks(i) == foi || freqPeaks(i) == foi+1
                y = [y s];
                pp = [pp freqPeaks(i)];  
           end
        end
    end
end
function printHTML(emg, eog, peaks, pf, ac, el, W, Sps, St, HTMLtitle, cond, freq, blockSize)
    output = strcat('SMART_',cond);
    if exist(output,'dir') ~= 7
       fprintf(1, '\nArtifacts dir doesn''t exists, Creating one...');
       mkdir(output);
    else 
       [succ] = rmdir(output,'s');
       mkdir(output);
       fprintf(1, '\nArtifacts dir already exists, Overwriting %s (%d)...',output,succ); 
    end
    cd(output);
    A = inv(W);

    % Determining powerspectra of the Original Data
    NFFT = blockSize;
    Hs = spectrum.welch({'Hann','periodic'}, NFFT, 50);
    Xorig = A*St;
    psXorig = [];
    for i =1:1:size(Xorig,1) 
        ps = psd(Hs, Xorig(i,:),'Fs',hdr.Fs);
        psXorig(i,:) = ps.Data'; 
    end
    meanPsXorig = mean(psXorig);
    emg_checked = [];
    
    % Printing images
    for i = 1:1:length(emg)
        psX = [];
        X = A(:,setdiff(1:size(Xorig,1),emg(i)))*St(setdiff(1:size(Xorig,1),emg(i)),:);
        for ii =1:1:size(X,1) 
            ps = psd(Hs, X(ii,:),'Fs',hdr.Fs);
            psX(ii,:) = ps.Data'; 
        end
        meanPsX = mean(psX); 
        dif = sum(abs(log(meanPsX(1:40))-log(meanPsXorig(1:40))));
        
        scrsz = get(0,'ScreenSize'); figure('Visible','off','Position',[1 1 scrsz(3)*.75 scrsz(4)*.2]);
        subplot(1,6,1), topoplot(A(:,emg(i)), el, 'nosedir', '-Y', 'electrodes', 'off');
        subplot(1,6,2), semilogy(freq(1:130), (Sps(emg(i),1:130))); axis 'tight'; %title(strcat('Coeff:',num2str(coeff(emg(i))))); axis 'tight'; 
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        subplot(1,6,3), loglog(freq(1:200), Sps(emg(i),1:200)); axis 'tight'; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        cof = polyfit(log(freq(41:60)'), log(Sps(emg(i),41:60)), 1); 
        if cof(1) > 0
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','r');
        else
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','b');
        end
        subplot(1,6,4), plot(ac(emg(i),:),'b+');  axis([1 64 -.2 1]); hold on; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        if mean(ac(emg(i),:)) < .4 && dif < 1
           title(strcat('min1to20lags:',num2str(min(ac(emg(i),1:20)))),'Color','r'); 
           set(gcf,'Color','c');
           emg_checked = [emg_checked, emg(i)];               
        else
           title(strcat('min1to20lags:',num2str(min(ac(emg(i),1:20)))),'Color','b');  
        end
       
        subplot(1,6,5), semilogy(ps.frequencies(1:60),meanPsXorig(1:60),'b'); hold on; semilogy(ps.frequencies(1:60),meanPsX(1:60),'r'); axis 'tight';
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        if dif <= 1
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','r');                
        else
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','b');        
        end
        subplot(1,6,6), plot(St(emg(i),1:10*blockSize)+max(St(emg(i),:)),'b'); hold on;
        plot(St(emg(i),10*blockSize+1:20*blockSize)+2*max(St(emg(i),:)),'r');
        plot(St(emg(i),20*blockSize+1:30*blockSize)+3*max(St(emg(i),:)),'k');
        plot(St(emg(i),30*blockSize+1:40*blockSize)+4*max(St(emg(i),:)),'m');
        plot(St(emg(i),40*blockSize+1:50*blockSize)+5*max(St(emg(i),:)),'c');
        plot(St(emg(i),50*blockSize+1:60*blockSize)+6*max(St(emg(i),:)),'g'); axis tight; 
        rms_ = sqrt(sum(diff(St(emg(i),1:blockSize:end)).^2)/length(St(emg(i),1:blockSize:end)));
        title(strcat('RMS = ',num2str(rms_)));         
        set(gcf,'PaperPositionMode','auto'); set(gcf, 'InvertHardCopy', 'off');
        saveas(gcf, strcat('emg_',num2str(emg(i))), 'tiff');
        close all;
        %cropping extra white space.
        imgs = imread(strcat('emg_',num2str(emg(i)),'.tif')); imwrite(imgs(:,300:end-150,:),strcat('emg_',num2str(emg(i)),'.tif'));
    end
    
    eog = union(eog, peaks(find(pf == 2))); % merging .5 Hz freq peaks and eog
    peaks = setdiff(peaks,peaks(find(pf==2)));
    pf = pf(find(pf~=2));
    eog_checked = [];
    for i = 1:1:length(eog)
        scrsz = get(0,'ScreenSize'); figure('Visible','off','Position',[1 1 scrsz(3)*.75 scrsz(4)*.2]);
        subplot(1,6,1), topoplot(A(:,eog(i)), el, 'nosedir', '-Y', 'electrodes', 'off');
        subplot(1,6,2), semilogy(freq(1:130), (Sps(eog(i),1:130)));  axis 'tight';%title(strcat('Coeff:',num2str(coeff(eog(i))))); axis 'tight'; 
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        subplot(1,6,3), loglog(freq(1:200), Sps(eog(i),1:200)); axis 'tight'; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        cof = polyfit(log(freq(41:60)'), log(Sps(eog(i),41:60)), 1); 
        if cof(1) > 0
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','r');
        else
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','k');
        end
        subplot(1,6,4), plot(ac(eog(i),:),'b+'); axis([1 64 -.2 1]);  hold on; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        if mean(ac(eog(i),:)) < .4
           title(strcat('mean:',num2str(mean(ac(eog(i),:)))),'Color','r');  % showing mean since EOG is classified using mean
        else
           title(strcat('mean:',num2str(mean(ac(eog(i),:)))),'Color','k');  
        end        
        psX = [];
        X = A(:,setdiff(1:size(Xorig,1),eog(i)))*St(setdiff(1:size(Xorig,1),eog(i)),:);
        for ii =1:1:size(X,1)      % LETS include frontal channels only for this.
            ps = psd(Hs, X(ii,:),'Fs',hdr.Fs);
            psX(ii,:) = ps.Data'; 
        end
        meanPsX = mean(psX);        % leaving out Z channels.
        subplot(1,6,5), semilogy(ps.frequencies(1:21),meanPsXorig(1:21),'b.-'); hold on; semilogy(ps.frequencies(1:21),meanPsX(1:21),'r.-'); axis 'tight';
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        dif = sum(abs(log(meanPsX(3:13))-log(meanPsXorig(3:13))))/sum(abs(log(meanPsXorig(3:13))));  % difference here for less than 6Hz.
        dif01 = sum(abs(log(meanPsX(1:2))-log(meanPsXorig(1:2))))/sum(abs(log(meanPsXorig(1:2))));  % difference here for less than 6Hz.
        dif720 = sum(abs(log(meanPsX(14:41))-log(meanPsXorig(14:41))))/sum(abs(log(meanPsXorig(14:41))));  % difference here for less than 6Hz.
        
        if mean(ac(eog(i),:)) > 0.9 && (dif >= 0.01 || dif01 >= 0.01)         %%%%%%greater than 1% change%%%%%%%
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','r'); 
            xlabel(strcat('d01=',num2str(dif01),'_d720=',num2str(dif720)));
            eog_checked = [eog_checked, eog(i)];
            set(gcf,'Color','y');  
        elseif mean(ac(eog(i),:)) < 0.9 && (dif >= 0.01 || dif01 >= 0.01)  
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','w'); 
            xlabel(strcat('d01=',num2str(dif01),'_d720=',num2str(dif720)),'Color','w');
            set(gcf,'Color','m');              
        else
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','w');        
            xlabel(strcat('d01=',num2str(dif01),'_d720=',num2str(dif720)),'Color','w');
            set(gcf,'Color','b');         %%%%%ignore these%%
        end

        subplot(1,6,6), plot(St(eog(i),1:10*blockSize)+max(St(eog(i),:)),'b'); hold on;
        plot(St(eog(i),10*blockSize+1:20*blockSize)+2*max(St(eog(i),:)),'r');
        plot(St(eog(i),20*blockSize+1:30*blockSize)+3*max(St(eog(i),:)),'k');
        plot(St(eog(i),30*blockSize+1:40*blockSize)+4*max(St(eog(i),:)),'m');
        plot(St(eog(i),40*blockSize+1:50*blockSize)+5*max(St(eog(i),:)),'c');
        plot(St(eog(i),50*blockSize+1:60*blockSize)+6*max(St(eog(i),:)),'g'); axis tight; 
        rms_ = max(St(eog(i),:)); %sqrt(sum(diff(St(eog(i),1:200:end)).^2)/length(St(eog(i),1:200:end))); %%%%trying 100ms (200) here%%
        title(strcat('RMS = ',num2str(rms_)));         

        set(gcf,'PaperPositionMode','auto'); set(gcf, 'InvertHardCopy', 'off');
        saveas(gcf, strcat('eog_',num2str(eog(i))), 'tiff');        
        close all;

        %cropping extra white space.
        imgs = imread(strcat('eog_',num2str(eog(i)),'.tif')); imwrite(imgs(:,300:end-150,:),strcat('eog_',num2str(eog(i)),'.tif'));
        
    end
    peaks_checked = [];
    for i = 1:1:length(peaks)
        scrsz = get(0,'ScreenSize'); figure('Visible','off','Position',[1 1 scrsz(3)*.75 scrsz(4)*.2]);
        subplot(1,6,1), topoplot(A(:,peaks(i)), el, 'nosedir', '-Y', 'electrodes', 'off');
        subplot(1,6,2), semilogy(freq(1:130), (Sps(peaks(i),1:130))); axis 'tight';
        if pf(i) == 3 || pf(i) == 33
            title(strcat('Peak at:',num2str((pf(i)-1)/2)),'Color','r'); axis 'tight'; 
            peaks_checked = [peaks_checked, peaks(i)];
            set(gcf,'Color','g');
        else
            title(strcat('Peak at:',num2str((pf(i)-1)/2))); axis 'tight'; 
        end
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        subplot(1,6,3), loglog(freq(1:200), Sps(peaks(i),1:200)); axis 'tight'; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        cof = polyfit(log(freq(41:60)'), log(Sps(peaks(i),41:60)), 1); 
        if cof(1) > 0
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','r');
        else
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','b');
        end        
        
        subplot(1,6,4), plot(ac(peaks(i),:),'b+'); axis([1 64 -.2 1]);  hold on; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        if mean(ac(peaks(i),:)) < .4  % same as threshold above.
           title(strcat('min1to20lags:',num2str(min(ac(peaks(i),1:20)))),'Color','r'); 
           peaks_checked = [peaks_checked, peaks(i)];
           set(gcf,'Color','g');
        else
           title(strcat('min1to20lags:',num2str(min(ac(peaks(i),1:20)))),'Color','b');  
        end        
        psX = [];
        X = A(:,setdiff(1:size(Xorig,1),peaks(i)))*St(setdiff(1:size(Xorig,1),peaks(i)),:);
        for ii =1:1:size(X,1) 
            ps = psd(Hs, X(ii,:),'Fs',hdr.Fs);
            psX(ii,:) = ps.Data'; 
        end
        meanPsX = mean(psX);        
        subplot(1,6,5), semilogy(ps.frequencies(1:130),meanPsXorig(1:130),'b'); hold on; semilogy(ps.frequencies(1:130),meanPsX(1:130),'r'); axis 'tight';
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        dif = sum(abs(log(meanPsX(1:120))-log(meanPsXorig(1:120)))); % diff should be calculated only at peaks.
        if dif >= 1
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','r'); 
        else
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','b');        
        end
        subplot(1,6,6), plot(St(peaks(i),1:10*blockSize)+1*max(St(peaks(i),:)),'b'); hold on;
        plot(St(peaks(i),10*blockSize+1:20*blockSize)+2*max(St(peaks(i),:)),'r');
        plot(St(peaks(i),20*blockSize+1:30*blockSize)+3*max(St(peaks(i),:)),'k');
        plot(St(peaks(i),30*blockSize+1:40*blockSize)+4*max(St(peaks(i),:)),'m');
        plot(St(peaks(i),40*blockSize+1:50*blockSize)+5*max(St(peaks(i),:)),'c');
        plot(St(peaks(i),50*blockSize+1:60*blockSize)+6*max(St(peaks(i),:)),'g'); axis tight; 
        rms_ = sqrt(sum(diff(St(peaks(i),1:blockSize:end)).^2)/length(St(peaks(i),1:blockSize:end)));
        title(strcat('RMS = ',num2str(rms_)));         
        set(gcf,'PaperPositionMode','auto'); set(gcf, 'InvertHardCopy', 'off');
        saveas(gcf, strcat('peaks_',num2str(peaks(i))), 'tiff');        
        close all;
        %cropping extra white space.
        imgs = imread(strcat('peaks_',num2str(peaks(i)),'.tif')); imwrite(imgs(:,300:end-150,:),strcat('peaks_',num2str(peaks(i)),'.tif'));
        
    end    
    normal_checked = [];
    normal = setdiff(1:size(St,1),union(union(emg,peaks),eog));
    for i = 1:1:length(normal)
        scrsz = get(0,'ScreenSize'); figure('Visible','off','Position',[1 1 scrsz(3)*.75 scrsz(4)*.2]);
        subplot(1,6,1), topoplot(A(:,normal(i)), el, 'nosedir', '-Y', 'electrodes', 'off');
        subplot(1,6,2), semilogy(freq(1:130), (Sps(normal(i),1:130)));axis 'tight'; %title(strcat('Coeff:',num2str(coeff(normal(i))))); axis 'tight';
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        subplot(1,6,3), loglog(freq(1:200), Sps(normal(i),1:200)); axis 'tight'; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        cof = polyfit(log(freq(41:60)'), log(Sps(normal(i),41:60)), 1); 
        if cof(1) > 0
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','r');
        else
            title(strcat('LLLC20-30Hz:',num2str(cof(1))),'Color','b');
        end        
        subplot(1,6,4), plot(ac(normal(i),:),'b+'); axis([1 64 -.2 1]);  hold on; set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        if mean(ac(normal(i),:)) < .4 
           title(strcat('mean:',num2str(mean(ac(normal(i),:)))),'Color','r'); 
           set(gcf,'Color','c');
        else
           title(strcat('mean:',num2str(mean(ac(normal(i),:)))),'Color','b');  
        end        
        psX = [];
        X = A(:,setdiff(1:size(Xorig,1),normal(i)))*St(setdiff(1:size(Xorig,1),normal(i)),:);
        for ii =1:1:size(X,1) 
            ps = psd(Hs, X(ii,:),'Fs',hdr.Fs);
            psX(ii,:) = ps.Data'; 
        end
        meanPsX = mean(psX);        
        subplot(1,6,5), semilogy(ps.frequencies(1:60),meanPsXorig(1:60),'b'); hold on; semilogy(ps.frequencies(1:60),meanPsX(1:60),'r'); axis 'tight';
        set(gca,'YMinorTick','on'); set(gca,'XMinorTick','on');
        dif = sum(abs(log(meanPsX(1:200))-log(meanPsXorig(1:200))));
        if dif >= 1
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','r'); 
        else
            title(strcat('R(X),B(Xo),Dif=',num2str(dif)),'Color','b');        
        end
        subplot(1,6,6), plot(St(normal(i),1:10*blockSize)+1*max(St(normal(i),:)),'b'); hold on;
        plot(St(normal(i),10*blockSize+1:20*blockSize)+2*max(St(normal(i),:)),'r');
        plot(St(normal(i),20*blockSize+1:30*blockSize)+3*max(St(normal(i),:)),'k');
        plot(St(normal(i),30*blockSize+1:40*blockSize)+4*max(St(normal(i),:)),'m');
        plot(St(normal(i),40*blockSize+1:50*blockSize)+5*max(St(normal(i),:)),'c');
        plot(St(normal(i),50*blockSize+1:60*blockSize)+6*max(St(normal(i),:)),'g'); axis tight; 
        rms_ = sqrt(sum(diff(St(normal(i),1:blockSize:end)).^2)/length(St(normal(i),1:blockSize:end)));
        title(strcat('RMS = ',num2str(rms_)));             
        if rms_ > 0.1
            set(gcf,'Color','y');   
            normal_checked = [normal_checked, normal(i)];            
        end        
        set(gcf,'PaperPositionMode','auto'); set(gcf, 'InvertHardCopy', 'off');
        saveas(gcf, strcat('normal_',num2str(normal(i))), 'tiff');        
        close all;
        %cropping extra white space.
        imgs = imread(strcat('normal_',num2str(normal(i)),'.tif')); imwrite(imgs(:,300:end-150,:),strcat('normal_',num2str(normal(i)),'.tif'));
        
    end    
    
   % print the HTML file
    fid = fopen('ArtifactsHTML.html','w');
    fprintf(fid, '\n<HTML>\n<TITLE>%s</TITLE>\n<BODY>\n <form action="http://localhost/cgi-bin/poll.cgi" method="POST">\n<TABLE border=1>\n', HTMLtitle);
    fprintf(fid, '<input type="text" name="filename", value=%s>\n',strcat(HTMLtitle,'.out'));
    emg2 = [emg_checked setdiff(emg,emg_checked)];
    for i = 1:1:length(emg2)
        if ismember(emg2(i),emg_checked)
            fprintf(fid, '\n<TR><TD><P><H2 align=center>EMG(Norm/EMG/RNorm) Source = %s </H2> <input type="radio" name="%d" value="norm"><input type="radio" name="%d" value="emg" checked><input type="radio" name="%d" value="Rnorm">', num2str(emg2(i)),emg2(i),emg2(i),emg2(i));
        else
            fprintf(fid, '\n<TR><TD><P><H2 align=center>EMG(Norm/EMG/RNorm) Source = %s </H2> <input type="radio" name="%d" value="norm" checked><input type="radio" name="%d" value="emg"><input type="radio" name="%d" value="Rnorm">', num2str(emg2(i)),emg2(i),emg2(i),emg2(i));
        end
        fprintf(fid, '\n<IMG SRC=''%s''></TD></TR>', strcat('emg_',num2str(emg2(i)),'.tif'));        
    end

    peaks2 = [peaks_checked setdiff(peaks,peaks_checked)];
    for i = 1:1:length(peaks2)
        if ismember(peaks2(i),peaks_checked)
            fprintf(fid, '\n<TR><TD><P><H2 align=center>PEAKS(Norm/Peaks/RNorm) Source = %s </H2> <input type="radio" name="%d" value="norm"><input type="radio" name="%d" value="peaks" checked><input type="radio" name="%d" value="Rnorm">', num2str(peaks2(i)),peaks2(i), peaks2(i), peaks2(i));        
        else
            fprintf(fid, '\n<TR><TD><P><H2 align=center>PEAKS(Norm/Peaks/RNorm) Source = %s </H2> <input type="radio" name="%d" value="norm" checked><input type="radio" name="%d" value="peaks"><input type="radio" name="%d" value="Rnorm">', num2str(peaks2(i)),peaks2(i), peaks2(i), peaks2(i));                    
        end
        fprintf(fid, '\n<IMG SRC=''%s''></TD></TR>', strcat('peaks_',num2str(peaks2(i)),'.tif'));        
    end
    
    eog2 = [eog_checked setdiff(eog,eog_checked)];
    for i = 1:1:length(eog2)
        if ismember(eog2(i), eog_checked)
            fprintf(fid, '\n<TR><TD><P><H2 align=center>EOG(Norm/EOG/RNorm/Reog) Source = %s </H2><input type="radio" name="%d" value="norm"><input type="radio" name="%d" value="eog" checked><input type="radio" name="%d" value="Rnorm"><input type="radio" name="%d" value="Reog">', num2str(eog2(i)), eog2(i), eog2(i), eog2(i), eog2(i));        
        else
            fprintf(fid, '\n<TR><TD><P><H2 align=center>EOG(Norm/EOG/RNorm/Reog) Source = %s </H2><input type="radio" name="%d" value="norm" checked><input type="radio" name="%d" value="eog"><input type="radio" name="%d" value="Rnorm"><input type="radio" name="%d" value="Reog">', num2str(eog2(i)), eog2(i), eog2(i), eog2(i), eog2(i));        
        end
        fprintf(fid, '\n<IMG SRC=''%s''></TD></TR>', strcat('eog_',num2str(eog2(i)),'.tif'));        
    end
    

    normal2 = [normal_checked setdiff(normal,normal_checked)];
    for i = 1:1:length(normal2)
        if ismember(normal2(i), normal_checked)
            fprintf(fid, '\n<TR><TD><P><H2 align=center>NORMAL(Norm/EOG/RNorm) Source = %s </H2> <input type="radio" name="%d" value="norm"><input type="radio" name="%d" value="eog" checked><input type="radio" name="%d" value="Rnorm">', num2str(normal2(i)), normal2(i), normal2(i), normal2(i));        
        else
            fprintf(fid, '\n<TR><TD><P><H2 align=center>NORMAL(Norm/EMG/RNorm) Source = %s </H2> <input type="radio" name="%d" value="norm" checked><input type="radio" name="%d" value="emg"><input type="radio" name="%d" value="Rnorm">', num2str(normal2(i)), normal2(i), normal2(i), normal2(i));        
        end
        fprintf(fid, '\n<IMG SRC=''%s''></TD></TR>', strcat('normal_',num2str(normal2(i)),'.tif'));        
    end
    fprintf(fid, '\n</TABLE><input type="submit" value="Vote"></BODY></HTML>');
    
    fclose(fid);
end
